<!DOCTYPE html>
<html>
  <head>
    <title>{% block head_title %}{% endblock %}</title>
    <link rel="stylesheet" href="https://cdn.jsdelivr.net/npm/@exampledev/new.css@1/new.min.css">

    {% block extra_head %}
    {% endblock %}
  </head>
  <body>
    {% block body %}

    <em>Important: Authentication via OAuth based providers will not work until you <a href="/admin/socialaccount/socialapp/">setup proper applicaton credentials</a>.</em>

    {% if messages %}
    <div>
      <strong>Messages:</strong>
      <ul>
	{% for message in messages %}
	<li>{{message}}</li>
	{% endfor %}
      </ul>
    </div>
    {% endif %}

    <div>
      <strong>Menu:</strong>
      <ul>
	{% if user.is_authenticated %}
	<li><a href="{% url 'account_email' %}">Change E-mail</a></li>
	<li><a href="{% url 'socialaccount_connections' %}">Social Accounts</a></li>
	<li><a href="{% url 'account_logout' %}">Sign Out</a></li>
	{% else %}
	<li><a href="{% url 'account_login' %}">Sign In</a></li>
	<li><a href="{% url 'account_signup' %}">Sign Up</a></li>
	{% endif %}
      </ul>
    </div>
    {% block content %}
    {% endblock %}
    {% endblock %}
    {% block extra_body %}
    {% endblock %}
  </body>
</html>
